<?php

// Contemplate Flattener (flattener.php) version 1.0.2
// Copyright 1999-2003 Type A Multimedia Network (http://www.typea.net)
// This file may be used, modified, or redistributed under the terms
// of the open-source agreement contained in docs/license.txt


// initialize variables that we might use before setting

	$flatten = 0;
	$unflatten = 0;
	$input_file = "";
	$output_file = "";
	$stop = 0;
	$failures = 0;
	$result = "";
	$output = "";
	$page_name = "";
	$pre_render_files = 1;
	$this_failure = 0;
	$changed = 0;
	$field_marker = "";
	$read_page_definitions_from = "";
	$unconvert_error = "";
	$changed_files = "";
	$check_URL = "";
	$settings_used = "";
	$template = "";
	$latest_mod_date = 0;
	

// define some useful subroutines
	
	function convert_URLs($text,$name) {
		GLOBAL $result, $URL_rewriting_flag;

		// convert URLs like ../assembled/page.html
		$text = preg_replace("/$URL_rewriting_flag\//i", "flattened/", $text, -1);
		
		// convert URLs like ../contemplate/assembler.php?page=page
		$text = preg_replace("/contemplate\/assembler\.php\?page\=([^<>\s\'\"]+)([<>\s\'\"])/i", "flattened/$1.html$2", $text, -1);
		
		// convert URLs like ../contemplate/assembler.php?template=default&main=field,main,home
		if (preg_match("/contemplate\/assembler\.php\?(template\=[^<>\s\'\"]+)[<>\s\'\"]/i", $text, $matches)) {
			$dynamic_page = $matches[1];
			$page_name = get_page_name($dynamic_page);
			if ($page_name != "") {
				$text = preg_replace("/contemplate\/assembler\/.php\?(template\=[^<>\s\'\"]+)([<>\s\'\"])/i", "flattened\/$page_name\.html$2", $text, -1);
			} else {
				$result .= "<font color=red>Error: Can't find a page name for $dynamic_page in $name.</font><br>";
			}
		}
	
		return $text;
	}

	function unconvert_URLs($text,$format_sample) {
		GLOBAL $unconvert_error, $URL_rewriting_flag;
		$flat_path = "../flattened/";
		$dynamic_path_short = "../$URL_rewriting_flag/";
		$dynamic_path_medium = "../contemplate/assembler.php?page=";
		if ((stristr($format_sample, $dynamic_path_short))&&(stristr($format_sample, $dynamic_path_medium))) {
			$unconvert_error = "Error: Assembled page may only contain short or medium format dynamic URLs, not both<br>";
		} else {
			if (stristr($format_sample, $dynamic_path_short)) {
				// convert flattened URLs to ../assembled/page.html
				$text = stri_replace($flat_path, $dynamic_path_short, $text);
			} elseif (stristr($format_sample, $dynamic_path_medium)) {
				// convert flattened URLs to ../contemplate/assembler.php?page=page
				$text = preg_replace("/".preg_quote($flat_path, "/")."([^<>\s\'\"]+)\.html/i", $dynamic_path_medium."$1", $text, -1);
			}
		}
		return $text;
	}
	
	
	function get_file($file, $method, $check_last_modified) { // method can be "http", "filesystem", or "auto" (auto looks at $pre_render_files list)
		GLOBAL $output, $file_missing, $pre_render_files, $script_location;
		GLOBAL $template, $read_page_definitions_from, $latest_mod_date, $latest_mod_date_formatted;
		GLOBAL $check_URL;
		$text = "";
		if (preg_match("/\/(\.\w.*)/", $file)) { // don't read files whose names start with "."
			$text = show_error("Not authorized to read the hidden file named \"$matches[1]\"");
		} else {
			
			// first, figure out if we're going to read over HTTP or through the filesystem...
			if (($method == "")||($method == "auto")) {
				if ($pre_render_files) {
					for ($n=0; $n < count($pre_render_files); $n++) {
						if (strstr($file, $pre_render_files[$n])) { $method = "http"; }
					}
				}
			}
			
			// ...then get the file
			if ($method == "http") {

				// 1) prepare the URL for this type of connection
				$file_URL = absolute_path($file, "http");

				// in PHP, the next three steps are the same for http or local connections, but it's easier to maintain the
				// different versions of this script if we lay out the conditional nesting the same way in every language
				// hence the repeated code here
				
				// 2) open a connection to the file (in PHP, steps 2, 3 and 5 are shared by the http and filesystem options, so the nesting is different here)
				@$myfile = fopen($file_URL, "r");
				if (! $myfile) {
					$text = show_error("Can't open the file named \"$file\" $check_URL");
					$file_missing = 1;
				} else {
				
					// 3) read the file
					while (!feof($myfile)) { $text .= fread($myfile, 4096); }
					$file_missing = 0;
					
					// 4) close the file connection
					fclose($myfile);
					
				}
	
				//$output .= "<p>reading $file_URL over http<p>";  // debug

			} else {

				// 1) prepare the URL for this type of connection
				$file_URL = absolute_path($file, "filesystem");
				
				// 2) open a connection to the file (in PHP, steps 2, 3 and 5 are shared by the http and filesystem options, so the nesting is different here)
				@$myfile = fopen($file_URL, "r");
				if (! $myfile) {
					$text = show_error("Can't open the file named \"$file\" $check_URL");
					$file_missing = 1;
				} else {
				
					// 3) read the file
					while (!feof($myfile)) { $text .= fread($myfile, 4096); }
					$file_missing = 0;
					
					// 4) for local files, catch the file's modification date (for $read_page_definitions_from and $template, plus content files when the template contains a env_last_modified tag)
					if ($check_last_modified) {
						$mtime = filemtime($file_URL);
						if ($mtime > $latest_mod_date) {
							$latest_mod_date = $mtime;
							$latest_mod_date_formatted = date("Y-m-d - g:i a", $mtime); // h for two-digit hour
						}
					}
	
					// 5) close the file connection
					fclose($myfile);
					
				}
	
				//$output .= "<p>reading $script_location_copy + $file_copy through filesystem<p>"; // debug

			}
				
			// finally, conform all line breaks to avoid cross-platform issues
			// we'd rather conform to Unix line breaks (\n), but that doesn't work in ASP, which doesn't support the s flag in regular expressions
			$text = str_replace("\r\n", "\r", $text); // change DOS to Mac
			$text = str_replace("\n", "\r", $text); // change Unix to Mac

		}
		return $text;
	}
	
	function get_setting($source_text, $setting, $allow_override, $allow_partial) {
		// allow most preferences to be overridden through the argument string
		GLOBAL $output, $settings_used;
		$value = "";
		$setting_no_spaces = $setting;
		$setting_no_spaces = preg_replace("/[ -]/", "_", $setting_no_spaces, -1);
		$settings_used .= "$setting_no_spaces,";
		
		$override = $_GET[$setting_no_spaces];
		if (($allow_override)&&($override != "")) {
			$value = $override;
		} else {
			$source_text = "\r".$source_text; // this makes it easy to make sure settings start on a new line
			if ($allow_partial) {  // this lets us use more verbose labels in the ini file
				if (preg_match("/[^\w]".$setting."[^:\r]*: ([^\r]*)\r*/i", $source_text, $matches)) {
					$value = $matches[1];
				}
			} else {  // this ensures an exact match in the pages file
				if (preg_match("/[^\w]".$setting.": ([^\r]*)\r*/i", $source_text, $matches)) {
					$value = $matches[1];
				}
			}
		}
		if ($value == "0") { $value = 0; } // this ensures that a 0 value will evaluate false in every language
		return $value;
	}
	
	function get_page_name($page_definition, $source) { // look for a page definition that exactly matches a given dynamic URL
		GLOBAL $output;
        $source = "\r".$source; // this makes it easier to match a page definition on the first line of the file
		$page_name = (preg_match("/[^\w](\w+)\:\s+".preg_quote($page_definition, "/")."[\&\s\r]/", $source, $matches)) ? $matches[1] : "" ;
		return $page_name;
	}
	
	function get_page_names($source) { // return a list of all the page names in a page definitions file
		GLOBAL $output;
		$page_names = Array();
		while (preg_match("/(\w+)\:\s+[\w]+/", $source, $matches)) {
			$this_page_name = $matches[1];
			array_push($page_names, $this_page_name);
			$source = str_replace($this_page_name.":", "", $source);
		}
		return $page_names;
	}

	function get_directory_listing($directory) { // return a list of all the items in the specified directory
		GLOBAL $output;
		if ($dir = opendir(absolute_path($directory, "filesystem"))) {
			$files = Array();
			while ($file = readdir($dir)) {
				if (substr($file, 0, 1) != ".") { array_push($files, $file); } // that gets files that don't start with a dot
			}  
			closedir($dir);
		}
		return $files;
	}

	function absolute_path($path, $method) { // convert a reference to a file within our site to an absolute URL or filesystem path
		GLOBAL $output, $script_location, $request, $URL_rewriting_flag;
		if ($script_location) { // if this hasn't been set yet, just return the original value

			if ($method == "filesystem") {
			
				$script_location_copy = $script_location;
				$file_copy = $path;
				while (strstr($file_copy, "../")) {
					// this avoids results like contemplate/../templates/default.html
					$script_location_copy = preg_replace("/[^\/]+\/$/", "", $script_location_copy, 1); // remove the last directory name from the script location
					$file_copy = preg_replace("/\.\.\//", "", $file_copy, 1); // remove the first updelim from the filename (in PHP, we need to use preg_replace so we only do one replacement)
				}
				$path = $script_location_copy.$file_copy;
				
			} else {
			
				$thisserver = $_SERVER["HTTP_HOST"];
				if (preg_match("/\/([^=]*\/)contemplate\//i", $request, $matches)) { $thispath = $matches[1]; }
				if (preg_match("/\/([^=]*\/)$URL_rewriting_flag\//i", $request, $matches)) { $thispath = $matches[1]; }
				
				// pass along the authentication information if present
				$login = ($_SERVER["PHP_AUTH_USER"]) ? $_SERVER["PHP_AUTH_USER"].":".$_SERVER["PHP_AUTH_PW"]."@" : "" ; 
				
				$path = str_replace("../", "http://$login$thisserver/$thispath", $path);
				
			}
		
		}
		return $path;
	}

	function stri_replace($pattern, $replacement, $subject) { // do a case-insensitive replace; this is a substitute for a PHP function that doesn't exist!
		GLOBAL $output;
		return preg_replace("/".preg_quote($pattern, "/")."/is", $replacement, $subject, -1);
	}

	function show_error($message) { // error display isn't as well developed here as in assembler, but we'll put this here so we can share more code
		return "Error: " + $message;
	}
	
	
// set up some primary variables

	$URL_rewriting_flag = "assembled"; // we'd rather store this in the preferences file, but we need it in order to read the preferences file!

	// figure out the location of the contemplate directory as shown in the browser, and use that as the basis for open commands;
	// this allows us to run the script from a symlink to the real contemplate folder, and paths will be relative to the
	// apparent, rather than the actual, location of assembler.php
	// this will also prevent problems if Windows calculates relative paths from the server root
	$request = (strstr($_SERVER["OS"], "Win")) ? $_SERVER["SCRIPT_NAME"] : ( (strpos($_SERVER["REQUEST_URI"], "?")) ? substr($_SERVER["REQUEST_URI"], 0, strpos($_SERVER["REQUEST_URI"], "?")) : $_SERVER["REQUEST_URI"] ) ;
	$script_location = stripslashes($_SERVER["PATH_TRANSLATED"]);
	$script_location = str_replace("\\", "/", $script_location);
	if (preg_match("/(.*\/)$URL_rewriting_flag\//i", $script_location, $matches)) {
		$script_location = $matches[1]."contemplate/";
	} else {
		$script_location = preg_replace("/(.*\/)[^\/]+/", "$1", $script_location, 1);
	}

	$flatten = $_POST['flatten'];
	$input_file = $_POST['input_file'];
	$output_file = $_POST['output_file'];
	$unflatten = $_POST['unflatten'];
		
	$successes = 0;
	$failures = 0;
	$changes = 0;


// read in a preferences (.ini) file if one exists

	$preferences = get_file("preferences.ini", "filesystem", 0);
	if ($preferences) {
		$field_marker = get_setting($preferences, "field marker", 1, 1);
		//$URL_rewriting_flag = get_setting($preferences, "URL rewriting flag", 1, 1);
		$read_page_definitions_from = get_setting($preferences, "read page definitions from", 1, 1);
	} else {
		$field_marker = "";
		//$URL_rewriting_flag = "";
		$read_page_definitions_from = "";
	}
	
	
// do a bit of processing on the settings, and set some defaults in case preferences.ini or individual settings are absent

	if ($field_marker == "") { $field_marker = "##"; }

	//if ($URL_rewriting_flag == "") { $URL_rewriting_flag = "assembled"; }

	if ($read_page_definitions_from == "") { $read_page_definitions_from = "pages.txt"; }


// set up the HTML page
	
	print "
	
		<html>
	
		<head>
		
			<title>Contemplate(tm) Web Content Manager</title>
			
			<link rel=stylesheet href=stylesheet.css type=text/css>
		
			<script language=JavaScript> login = 0; </script>
			<script language=JavaScript src=login/login.js></script>		
			<script language=JavaScript> if (!login) { location = '../index.html'; } </script>

		</head>
		
		<body bgcolor=#FFFFFF>
				
			<p class=title>Contemplate</p>
			<p class=body><a href=index.html>Utilities menu</a> &gt; Flattener</p><br>
	
	";
	

// require a current version of the extractor before proceeding

	$required_version = 1.0;
	$extractor_text = get_file("assembler.php", "filesystem", 0);
	
	if (preg_match("/version (\d+\.\d+)/", $extractor_text, $matches)) {
		$installed_version = $matches[1];
		if ($installed_version < $required_version) {
			print "Error: This version of the Flattener requires version $required_version or newer of Contemplate (assembler.php).<p>";
			$stop = 1;
		}
	}
	

// present some options on the first run
	
	if ((!$flatten)&&(!$input_file)&&(!$unflatten)&&(!$stop)) {

	/////
	// show the intro page
	/////
	
		print "
			
			<p class=subheading>Flatten all dynamic pages</p>
			<p class=body>Before you do this, create a folder called \"flattened\" at the root level of your website, and give it world-write access (chmod a+w flattened)</p>
			
			<form name=flatten method=post>
			
				<p class=body><input type=submit value=Flatten></p><br>
				
				<input type=hidden name=flatten value=1>
				
			</form>
			
			<p class=subheading>Parse individual files</p>
			<p class=body>Use this option to convert a file that contains links to dynamic pages, even if the file itself isn't dynamically generated<br>
			Enter paths relative to the site root</p>
			
			<form name=parse method=post>
			
				<table><tr valign=bottom><td>
				
					<p class=body>Input file<br>
					<input type=text name=input_file></p>
				
				</td><td>
				
					<p class=body>Output file<br>
					<input type=text name=output_file></p>
				
				</td></tr></table>
				
				<p class=body><input type=submit value=Parse></p><br>
			
			</form>
		
			<p class=subheading>Unflatten a flattened site</p>
			<p class=body>Use this option to update the dynamic files from which a flattened site was generated if the flattened files have been edited<br>
			Before you do this, give world-write access to all content files (chmod -R a+w content)</p>
			
			<form name=unflatten method=post>
			
				<p class=body><input type=submit value=Unflatten><input type=hidden name=unflatten value=1></p><br>
			
			</form>
			
			<div class=body>
			
		";
		

	} else {
	
	
	// grab the text of the page definitions file; both options will use it
	
		$pages_file = get_file($read_page_definitions_from, "filesystem", 0);
		
	
		if ($flatten) {
		
		////
		// flatten the site
		////
		
			// parse the pages file to get an array of page names and an array of URLs
			
				$pages = Array(); $URLs = Array();
				$page_names = get_page_names($pages_file);
				for ($i=0; $i<count($page_names); $i++) {
					$this_page_name = $page_names[$i];
					$this_page = "../flattened/$this_page_name.html";
					$this_URL = "../contemplate/assembler.php?page=$this_page_name&skip_rendering=1";
					$pages[$i] = $this_page;
					$URLs[$i] = $this_URL;
				}
			
			// open each URL, change its dynamic links to static ones, and save it to its page name
			
				for ($i=0; $i<count($pages); $i++) {
				
					// get the dynamic page
					$file_missing = 0;
					$page_rendered = get_file($URLs[$i], "http", 0);
					
					if ($file_missing) {
						$result .= "<font color=red>$page_rendered</font><br>"; // that will contain the error if $file_missing == 1
						$failures++;
					} else {
					
						// convert all the dynamic URLs to flattened ones
						$page_rendered = convert_URLs($page_rendered, $pages[$i]);
						
						// save the flattened page
						@$my_file = fopen(absolute_path($pages[$i], "filesystem"), "w");
						if (!$my_file) {
							$result .= "<font color=red>Error: Can't open the file $pages[$i] for flattening.</font><br>";
							$failures++;
						} else {
							fwrite ($my_file, $page_rendered);
							$result .= "Processing $pages[$i]<br>";
							$successes++;
							fclose ($my_file);
						}
						
					}
					
					// provide feedback as the script runs
					print $result;
					$result = "";
				
				}
		
			// show success rate and, if errors, offer tips
		
				print "<br><b>Created $successes files and encountered $failures errors</b>";
		
	
		} elseif ($input_file) {
	
		/////
		// parse a single file
		////
		
			// open the file, change its dynamic links to static ones, and save it to its output location
			
				// in case the paths aren't relative to the contemplate folder
				$input_file = stristr($input_file, "../") ? $input_file : "../".$input_file ;
				$output_file = stristr($output_file, "../") ? $output_file : "../".$output_file ;
				
				// get the file
				$delimiter = (!strstr($input_file, "?")) ? "?" : "&" ;
				$file = get_file($input_file.$delimiter."skip_rendering=1", "http", 0);

				// convert all the dynamic URLs to flattened ones
				$file = convert_URLs($file, $input_file);
			
				// save the parsed file
				@$my_file = fopen(absolute_path($output_file, "filesystem"), "w");
				if (!$my_file) {
					$failures++;
				} else {
					fwrite($my_file, $file);
					fclose ($my_file);
				}
				
			// show result
		
				if (!$failures) {
					print "<b>Converted $input_file successfully</b><p>";
				} else {
					print "Error: Couldn't open the file $output_file for saving.<br>";
					print "<b>Failed to convert $input_file</b><p>";
				}
				
		
		} elseif ($unflatten) {
	
		/////
		// move changes to the flattened files back into the dynamic files
		/////
		
			// get a list of flat pages
			
				$flat_files = get_directory_listing("../flattened/");
						
			// open assembler.php?page=$page&get_info=1 for each flat page, but skip if there's a flattener error
			
				//$flat_files = Array ("field_1.html"); // for testing one file at a time
				
				$dynamic_files = Array();
				for ($i=0; $i<count($flat_files); $i++) {
				
					$flat_files[$i] = "../flattened/".$flat_files[$i];
					
					$dynamic_files[$i] = "../contemplate/assembler.php?page=".$flat_files[$i];
					$dynamic_files[$i] = str_replace("page=../flattened/","page=",$dynamic_files[$i]);
					$dynamic_files[$i] = preg_replace("/\.\w+$/", "", $dynamic_files[$i], 1);
									
					$flat_file = get_file($flat_files[$i], "filesystem", 0);
					$dynamic_file = get_file($dynamic_files[$i]."&get_info=1", "http", 0);
					
					if (stristr($dynamic_file, "Flattener error")) {
						print "<font color=red>Error: Couldn't find dynamic page for $flat_files[$i]</font><br>";
						$failures++;
					} else {
						print "<br>Processing file $dynamic_files[$i]<br>";
						
						// something about the last character of each file prevented a match to the last template piece, so strip them off now
						$dynamic_file = preg_replace("/(.*).(<\/template_piece-.*)/", "$1"."$2", $dynamic_file, 1);
						$flat_file = substr($flat_file, 0, -1);
						
						// convert flattened URLs in the flat file back to their dynamic counterparts for comparison
						$flat_file = unconvert_URLs($flat_file, $dynamic_file); // select one or the other URL option based on what's in the dynamic file
						if ($unconvert_error) {
							print "<font color=red>$unconvert_error</font>";
							$failures++;
						} else {
						
							// loop through the file info and collect all the template pieces
							$j = 1; $template_pieces = Array();
							while (preg_match("/<template_piece-$j>(.*)<\/template_piece-$j>/", $dynamic_file, $matches)) {
								$template_pieces[($j-1)] = $matches[1];
								$j++;
							}
							
							// loop through template pieces and parse flat and dynamic files to make two arrays of content pieces
							$flat_file_copy = $flat_file;
							$flat_pieces = Array();
							$dynamic_pieces = Array();
							for ($j=0; $j<(count($template_pieces)-1); $j++) {
								
								$changed = 0;
								$j_plus = $j+1; $j_plus_plus = $j+2;
								$next_piece = "flattener-no-match-flag";
								$temp = substr($flat_file_copy, 0, strlen($template_pieces[$j]));
								if ($temp == $template_pieces[$j]) {

									// strip the first template piece right off
									$flat_file_copy = substr($flat_file_copy, strlen($template_pieces[$j]));
									
									// then scan for a match for the second template piece
									$range = strlen($flat_file_copy) - strlen($template_pieces[$j_plus]);
									for ($k=0; $k<=$range; $k++) {
										if (substr($flat_file_copy, $k, strlen($template_pieces[$j_plus])) == $template_pieces[$j_plus]) {
											$next_piece = substr($flat_file_copy, 0, $k);
											$flat_file_copy = substr($flat_file_copy, $k, strlen($flat_file_copy));
											$k = $range + 1;
										}
									}
									if ($next_piece == "flattener-no-match-flag") {
										$changed = 2;
									}
								} else {
									$changed = 1;
								}
								if ($changed) {
								//all the preceding lines except $j_plus = $j+1 could be avoided with this line, but it doesn't quite work -- too greedy?
								//if ($flat_file_copy !~ /\Q@template_pieces[$j]\E(.*?)(\Q@template_pieces[$j_plus]\E.*)/) {
									print "<font color=red>Error: Template piece ".($j+$changed)." has been changed in $flat_files[$i]</font><br>";
									print "Hint: If using conditional tags, check to see that all conditions are the same as when flattened<br>";
									//print "\r\r<template_piece $j_plus>$template_pieces[$j]</template_piece $j_plus>"; // debug
									//print "\r\r<template_piece $j_plus_plus>$template_pieces[$j_plus]</template_piece $j_plus_plus>"; // debug
									//print "\r\r<$flat_files[$i]>$flat_file_copy</$flat_files[$i]>"; // debug
									$failures++;
									$j = count($template_pieces); // stop processing this file
								} else {
								
									$flat_pieces[$j] = $next_piece; // $1
									
									if (preg_match("/<content-$j_plus>(.*)<\/content-$j_plus>/", $dynamic_file, $matches)) {
										$dynamic_pieces[$j] = $matches[1];
									}

									// if content from flat file and dynamic file are different, open the content file and replace dynamic with flat
									if ($flat_pieces[$j] == $dynamic_pieces[$j]) {
										print "Piece $j_plus is the same<br>";
									} else {
											
										// don't bother with certain embed types
										if (preg_match("/<embed_type-$j_plus>random<\/embed_type-$j_plus>/", $dynamic_file)) {
											print "Skipping random content in piece $j_plus<br>";
										} elseif (preg_match("/<embed_type-$j_plus>env_.+<\/embed_type-$j_plus>/", $dynamic_file)) {
											print "Skipping environment variable in piece $j_plus<br>";
										} elseif (preg_match("/<embed_type-$j_plus>passthr.+<\/embed_type-$j_plus>/", $dynamic_file)) {
											print "Skipping passthru content in piece $j_plus<br>";
										} else {

											print "<font color=green>Updating piece $j_plus</font><br>";
											
											// open, update, and save
											$this_file = ""; $this_failure = 0;
											if (preg_match("/<content_file-$j_plus>(.*)<\/content_file-$j_plus>/",$dynamic_file,$matches)) {
												$this_file = $matches[1];
												$content = get_file($this_file, "filesystem", 0);
												
												$content = str_replace($dynamic_pieces[$j], $flat_pieces[$j], $content);
												
												@$my_file = fopen(absolute_path($this_file, "filesystem"), "w");
												if ($my_file) {
													fwrite ($my_file, $content);
													if (!stristr($changed_files, $this_file)) {
														$changed_files .= $this_file.",";
													}
													fclose ($my_file);
												} else {
													$this_failure++;
												}
											} else {
												$this_failure++;
											}

											if ($this_failure) {
												if (!$this_file) { $this_file = "(unknown)"; }
												print "<font color=red>Error: Couldn't save changes to content file $this_file</font><br>";
												$failures++;
											} else {
												$changes++;
											}
											
											//print "<flat piece>@flat_pieces[$j]</flat piece><p>\r\r"; // debug
											//print "<dynamic piece>@dynamic_pieces[$j]</dynamic piece><p>\r\r"; // debug
											//print "<flat file>$flat_file</flat file><p>\r\r"; // debug
											//print "<dynamic file>$dynamic_file</dynamic file><p>\r\r"; // debug
										
										}
										
									}
								
								}
								
							}
									
						}
					
					}
				
				}
				
			// show result
		
				if (strlen($changed_files) > 0) {
					$changed_files = substr($changed_files, 0, -1);
					$changed_files = explode(",", $changed_files);
					$changed_files_count = count($changed_files);
				} else {
					$changed_files_count = 0;
				}
				print "<br><b>Updated $changes fields in $changed_files_count files with $failures errors</b><p>";
				
		}
	
		print "</div>";

		if (!$stop) {
			print "<p class=body><a href=JavaScript:history.back()>Back</a></p>";
		}

		print $output; // we don't use the $output variable in this script, but this helps us share code with assembler.php
				

// finish off a big conditional up there
	
	}	
	
	
// we're done!

	print "</body></html>";

?>
